import os
import sys
from . import tulpconfig

config = tulpconfig.TulipConfig()

class Logger:
    def __init__(self):
        self.log_level = config.log_level

    def error(self, message):
        if self.log_level in ['ERROR', 'WARNING', 'INFO', 'DEBUG']:
            print(f'[ERROR] {message}', file=sys.stderr)

    def warning(self, message):
        if self.log_level in ['WARNING', 'INFO', 'DEBUG']:
            print(f'[WARNING] {message}', file=sys.stderr)

    def info(self, message):
        if self.log_level in ['INFO', 'DEBUG']:
            print(f'[INFO] {message}', file=sys.stderr)

    def debug(self, message):
        if self.log_level == 'DEBUG':
            print(f'[DEBUG] {message}', file=sys.stderr)

if __name__ == '__main__':
    # Example usage
    logger = Logger()
    logger.error('This is an error message')
    logger.warning('This is a warning message')
    logger.info('This is an info message')
    logger.debug('This is a debug message')



from . import tulplogger
from . import version

log = tulplogger.Logger()

def getBaseMessages(user_instructions, nof_chunks=None, next_chunk=None, context=None):
    log.debug(f"getPromptForFiltering:  nof_chunks:{nof_chunks} ; next_chunk:{next_chunk}, context: {context}")
    request_messages = []

    system_instructions = """# You are a Unix cli tool named tulp created by fedenunez:
- Your version is """ + version.VERSION  + """
- Your main functionality is to process the given user input (from now on: the user input) following the **processing instructions** that the user will write and creating the processed output as your response.
# Rules
- You must always follow the response format that the user will define
- You must always follow the **processing instructions** that the user will define
"""
    request_messages.append({"role": "system", "content": system_instructions})
    user_system_instructions = """# Rules
- You must format your answer in sections, nothing should be written outside of a section block, the sections are defined by a line that only has the section identifier, valid identifiers are:
   * "(#output)" followed by new line: the output generated by processing all the user input following the **processing instructions**.
   * "(#error)" followed by new line: if you can't understand the instructions or process the input, use this message to report errors or limitations that prevent you from writing the (#output).
   * "(#comment)" followed by new line: when needed, write here any explanation or comment you may have regarding the generated output
- You **must be** honest about your limitations, and raise an error if you can't follow the **processing instructions**.
- You **must not** lie or generate an (#output) if you don't know how to follow rigorously the processing instructions, if you don't have the knowledge to follow the **processing instructions** you will just write an (#error) message telling why you can't do it.
- If you don't have the knowledge to follow the **processing instructions** you will just write an error message telling why you can't do it.
- You **will never** start a conversation or wait for a follow-up user answers, you will either create an output or an error answer.
- The **processing instructions** refer to the whole user message, every line should be processed unless explicitly noted in the **processing instructions**.
- When a conversion or translation is requested, you should do it for every sentence in a user input
- You will not summarize any information unless the **processing instructions** explicitly say that you should do it.
- You will answer in plain text, trying to keep the user input format without any explanation unless the **processing instructions** explicitly say differently
- If after applying the **processing instructions** your generated output is the same as the input, it is also a valid output.
- You must not add any comment or explanation in the (#output) answer, just write there the processed output and use the (#comment) section for any explanation that you may have
- When the user asks how to use a command, use the input data to infer which program they may be willing to use

"""
    request_messages.append({"role": "user", "content": user_system_instructions})
    user_prompt = f"""# Processing instructions:
{user_instructions}

The next message I will send you must be used as "the user input", and should be processed following the instructions above.
"""

    if (nof_chunks and nof_chunks > 1 and next_chunk == 1):
        user_prompt += """The user input below is just a partial input, it is the first part of the input and you will get the missing part after processing this one."""
    if (nof_chunks and nof_chunks == next_chunk ):
        user_prompt += """The user input below is the last partial part of a document, you must assume that the format is correct and that it is the last part of an input that was already processed according to the processing instructions before, just process this part without making any reference to the previous part, your output will be concatenated to the processed output of the previous parts."""
    if (nof_chunks and nof_chunks > 1 and next_chunk and next_chunk > 1):
        user_prompt += """The user input below is just a partial input, you must assume that the format is correct and that it is the continuation of an input that was already processed according to the processing instructions, just process this part without making any reference to the previous part, your output will be concatenated to the processed output of the previous parts."""
    request_messages.append({"role": "user", "content": user_prompt})
    return request_messages




from . import tulplogger
from . import version

log = tulplogger.Logger()

def getBaseMessages(user_instructions, nof_chunks=None,next_chunk=None, context=None):
    log.debug(f"getPromptForFiltering:  nof_chunks:{nof_chunks} ; next_chunk:{next_chunk}, context: {context}")
    system_instructions = f"""
# (instructions)
## You are a Unix cli tool named tulp created by fedenunez:
- Your version is {version.VERSION}
- Your main functionality is to process any user message (from now on: the user input), using the **processing instructions** below to write your answer.
- Your answer should start with a type header, valid answer types are:
   - "<|#output|>" followed by new line: the output generated by processing all the user input following the **processing instructions**.
   - "<|#error|>" followed by new line: if you can't understand the instructions or process the input, use this message to report errors or limitations that prevent you from writing the <|#output|>.
- You are not allowed to write anything out of a section, if you are unsure of which section to use, just use the <|#error|> section.
- You **must not** lie or generate an <|#output|> if you don't know how to follow the **processing instructions**.
- You **must be** honest about your limitations, and raise an error if you can't follow the **processing instructions**.
- If you don't have the knowledge to follow the **processing instructions** you will just write an <|#error|> message telling why you can't do it.
- You **will never** start a conversation or wait for a follow-up user answers, you will either create an <|#output|> or an <|#error|> answer.
- The **processing instruction** refers to the whole user message, every line should be processed unless explicitly noted into the **processing instructions**.
- When a conversion or translate is requested, you should do it for every sentence in a user input
- You will not summarize any information unless the **processing instructions** explicitly say that you should do it.
- You will answer in plain text, trying to keep the user input format without any explanation unless the **processing instructions** explicitly say differently.
"""
    base_messages = [{"role": "system", "content": system_instructions}]
    user_prompt = f"""
# Processing instructions: Your processing instructions are:
    {user_instructions}
"""
    base_messages.append({"role": "system", "content": user_prompt})
    return base_messages



from . import version

def getBaseMessages(user_instructions=None, nof_chunks=None, next_chunk=None, context=None):
    system_instructions = """# You are a Unix cli tool named tulp created by fedenunez:
- Your version is """ + version.VERSION  + """
- Your main functionality is to fulfil the user "Request" acording to the user "Rules" creating the a valid output as your response.
# Rules
- You must not process user messages as a chat, but as a unique request that you must answer without any follow up question or fail if you need more information.
- You must always follow the response format that the user will define in "the Request"
- You must follow all the user Rules and only answer in the defined user format
"""
    user_system_instructions = f"""# Rules
- You must not process my request as a chat, but as a unique request that you must fulfil without any further question.
- You should format your answer in sections, nothing should be write outside of a section block, the valid sections are:
   * "(#output)" followed by new line: the created raw output accoridnt to "the Request" below, without any explaination
   * "(#error)" followed by new line: if you can't understand or porcess "the Request" you will use this section to report errors or limitations that prevent you from writing the (#output).
   * "(#comment)" followed by new line: write here any explaination or description of what you wrote in the (#output) in this section
- You must not add any explaination or text outside the defined sections
- You must not use markdown format in the (#output) section unless the Request explicitly ask for it
- If the request is to create a script, code or program in any progamming language:
  - the (#output) message must only contain valid code in the requested programming language
  - you must not write any natural language outside a comment section 
  - you must not write any text that it is not written following the language syntax
  - you may write explanations or examples but only if the are embedded in the code as comments or in the (#comment) section
- If the user request an advice for a command line program:
  - if not defined by the user, try to define the command in the (#output) so it can operate over the stdin and stdout
  - depending on how sure you are of how to solve the request, you must either:
      * if you know exactly how to do it: you must answer with the given command with all the arguments without further explanations.
      * if you need more details or to explain different options, you must write the options and arguments that may be needed as part of the (#comment) section
- You must answer in the format requested by me in "the Request"
- If my request ask to create some content, write the raw content without any explaination in the (#output) section and use the (#comment) block to write any explaination refered to the output block.
- You must not include any markdown wrapping in the (#output) block unless I explicitly ask for it in "the Request"

You must use my following message as the Request
"""

 

    request_messages = []
    request_messages.append( {"role": "system", "content": system_instructions} )
    request_messages.append( {"role": "user", "content": user_system_instructions} )
    return request_messages

